package com.supervision.knowsub.controller;

import com.supervision.knowsub.exception.BusinessException;
import com.supervision.knowsub.service.FileService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;

@Slf4j
@RestController
@Tag(name = "文件管理类")
@RequiredArgsConstructor
@RequestMapping("file")
public class FileController {

    private final FileService fileService;

    @Operation(summary = "文件上传")
    @PostMapping("upload")
    public String fileUpload(@RequestParam(name = "file") MultipartFile file) throws IOException {
        byte[] bytes = file.getBytes();
        if (bytes.length > 20 * 1024 * 1024){
            throw new BusinessException("文件过大,文件大小请不要超过20MB");
        }
        return fileService.uploadFile(file.getOriginalFilename(), bytes);
    }

    @Operation(summary = "根据文件ID删除文件")
    @GetMapping("deleteFileById")
    public void deleteFileById(String fileId){
        fileService.deleteFileById(fileId);
    }

    @Operation(summary = "下载文件")
    @GetMapping("/downloadFile")
    public void downloadFile(@RequestParam String fileId, HttpServletResponse response) {

        fileService.downloadFile(fileId, response);

    }
}
