package com.supervision.knowsub.controller.knowledge;

import cn.hutool.core.lang.Assert;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.supervision.knowsub.entity.vo.knowledge.KnowledgeDetailResVO;
import com.supervision.knowsub.entity.vo.knowledge.ModifyKnowledgeReqVO;
import com.supervision.knowsub.entity.vo.knowledge.SaveKnowledgeReqVO;
import com.supervision.knowsub.service.KnowledgeManageService;
import com.supervision.knowsub.model.Knowledge;
import com.supervision.knowsub.vo.knowledge.KnowledgePageResVO;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.enums.ParameterIn;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDate;
import java.time.LocalDateTime;

@Slf4j
@RestController
@Tag(name = "知识库查询")
@RequiredArgsConstructor
@RequestMapping("knowledge")
public class KnowledgeManageController {

    private final KnowledgeManageService knowledgeManageService;

    @Operation(summary = "查询知识库分页-除全部可以看所有外,其他tab都是看到自己报送的知识的状态(或参与过的)")
    @Parameters({
            @Parameter(name = "status", description = "流转状态 为空查全部,1草稿 2通过 3驳回 4已撤回 5已失效 6已删除 9待审批(这里包含了10,11,12,统一用9)", in = ParameterIn.QUERY),
            @Parameter(name = "title", description = "标题,支持模糊", in = ParameterIn.QUERY),
            @Parameter(name = "publishDeptId", description = "发文部门id", in = ParameterIn.QUERY),
            @Parameter(name = "baseId", description = "应用字库ID", in = ParameterIn.QUERY),
            @Parameter(name = "submittedDeptId", description = "报送部门ID", in = ParameterIn.QUERY),
            @Parameter(name = "publishDateBegin", description = "发文时间开始", in = ParameterIn.QUERY),
            @Parameter(name = "publishDateEnd", description = "发文时间结束", in = ParameterIn.QUERY),
            @Parameter(name = "submitDateBegin", description = "报送时间开始", in = ParameterIn.QUERY),
            @Parameter(name = "submitDateEnd", description = "报送时间结束", in = ParameterIn.QUERY),
            @Parameter(name = "pageNum", description = "分页", in = ParameterIn.QUERY),
            @Parameter(name = "pageSize", description = "分页", in = ParameterIn.QUERY)
    })
    @GetMapping("queryKnowledgePage")
    public IPage<KnowledgePageResVO> queryKnowledgePage(@RequestParam(required = false) Integer status,
                                                        @RequestParam(required = false) String title,
                                                        @RequestParam(required = false) String publishDeptId,
                                                        @RequestParam(required = false) String baseId,
                                                        @RequestParam(required = false) String submittedDeptId,
                                                        @RequestParam(required = false) LocalDate publishDateBegin,
                                                        @RequestParam(required = false)LocalDate publishDateEnd,
                                                        @RequestParam(required = false) LocalDate submitDateBegin,
                                                        @RequestParam(required = false) LocalDate submitDateEnd,
                                                        Integer pageNum, Integer pageSize) {
        return knowledgeManageService.queryKnowledgePage(status, title, publishDeptId, baseId, submittedDeptId, publishDateBegin, publishDateEnd,submitDateBegin,submitDateEnd, pageNum, pageSize);
    }

    @Operation(summary = "新增知识库")
    @PostMapping("saveKnowledge")
    public void saveKnowledge(@RequestBody SaveKnowledgeReqVO reqVO) {
        knowledgeManageService.saveKnowledge(reqVO);
    }

    @Operation(summary = "修改知识库(status:1草稿/3驳回/4已撤回/5已失效,才能修改)")
    @PostMapping("updateKnowledge")
    public void updateKnowledge(@RequestBody ModifyKnowledgeReqVO reqVO) {
        Assert.notBlank(reqVO.getKnowledgeId(), "知识ID不能为空");
        knowledgeManageService.updateKnowledge(reqVO);
    }

    @Operation(summary = "撤回知识库(status:2通过/5已失效)")
    @GetMapping("recallKnowledge")
    public void recallKnowledge(String knowledgeId,String remark) {
        knowledgeManageService.recallKnowledge(knowledgeId,remark);
    }

    @Operation(summary = "查看知识的详细信息")
    @GetMapping("queryKnowledgeDetail")
    public KnowledgeDetailResVO queryKnowledgeDetail(String knowledgeId) {
        return knowledgeManageService.queryKnowledgeDetail(knowledgeId);
    }

    @Operation(summary = "删除知识库-逻辑删除(status:1草稿/2通过/3驳回/4已撤回/5已失效)")
    @GetMapping("deleteKnowledge")
    public void deleteKnowledge(String knowledgeId,String remark) {
        knowledgeManageService.deleteKnowledge(knowledgeId,remark);
    }


}
