package com.supervision.knowsub.service.impl;

import cn.hutool.core.lang.Assert;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.supervision.knowsub.model.SystemDept;
import com.supervision.knowsub.service.DeptManageService;
import com.supervision.knowsub.service.SystemDeptService;
import com.supervision.knowsub.service.UserManageService;
import com.supervision.knowsub.vo.dept.DeptInfoReqVo;
import com.supervision.knowsub.vo.dept.DeptInfoResVo;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
@RequiredArgsConstructor
public class DeptManageServiceImpl implements DeptManageService {

    private final SystemDeptService systemDeptService;

    private final UserManageService userManageService;
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String saveDept(DeptInfoReqVo deptInfoReqVo) {

        Assert.notEmpty(deptInfoReqVo.getDeptName(), "部门名称不能为空");
        Assert.notEmpty(deptInfoReqVo.getDeptCode(), "部门编码不能为空");

        Long count = systemDeptService.lambdaQuery().eq(SystemDept::getDeptCode, deptInfoReqVo.getDeptCode())
                .or(true,(queryWrapper -> queryWrapper.eq(SystemDept::getDeptName, deptInfoReqVo.getDeptName()))).count();
        Assert.isTrue(count == 0, "部门名或编码已存在");
        SystemDept systemDept = new SystemDept();
        systemDept.setDeptCode(deptInfoReqVo.getDeptCode());
        systemDept.setDeptName(deptInfoReqVo.getDeptName());
        systemDept.setParentDeptId(deptInfoReqVo.getParentDeptId());
        systemDept.setRemark(deptInfoReqVo.getRemark());

        systemDeptService.save(systemDept);

        return systemDept.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateDept(DeptInfoReqVo deptInfoReqVo) {

        Assert.notEmpty(deptInfoReqVo.getId(), "id不能为空");
        Assert.notEmpty(deptInfoReqVo.getDeptName(), "部门名称不能为空");
        Assert.notEmpty(deptInfoReqVo.getDeptCode(), "部门编码不能为空");

        Long count = systemDeptService.lambdaQuery()
                .and(true,(queryWrapper -> queryWrapper.eq(SystemDept::getDeptCode, deptInfoReqVo.getDeptCode())
                        .or().eq(SystemDept::getDeptName, deptInfoReqVo.getDeptName())))
                .and(true,(queryWrapper -> queryWrapper.ne(SystemDept::getId, deptInfoReqVo.getId())))
                .count();
        Assert.isTrue(count == 0, "部门名或编码已存在");

        SystemDept systemDept = new SystemDept();
        systemDept.setId(deptInfoReqVo.getId());
        systemDept.setDeptCode(deptInfoReqVo.getDeptCode());
        systemDept.setDeptName(deptInfoReqVo.getDeptName());
        systemDept.setParentDeptId(deptInfoReqVo.getParentDeptId());
        systemDept.setRemark(deptInfoReqVo.getRemark());

        systemDeptService.updateById(systemDept);

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean deleteDept(String id) {
        Long count = userManageService.countUserByDeptId(id, 0);
        Assert.isTrue(count == 0, "该部门下存在用户，不能删除");
        return systemDeptService.removeById(id);
    }

    @Override
    public IPage<DeptInfoResVo> pageListDept(String deptName, String deptCode, Integer pageNum, Integer pageSize) {
        return systemDeptService.pageListDept(deptName, deptCode, pageNum, pageSize);
    }
}
