package com.supervision.knowsub.service.impl;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.ObjUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.supervision.knowsub.model.SystemDic;
import com.supervision.knowsub.model.SystemDicData;
import com.supervision.knowsub.service.DicManageService;
import com.supervision.knowsub.service.SystemDicDataService;
import com.supervision.knowsub.service.SystemDicService;
import com.supervision.knowsub.vo.dic.DicDataReqVo;
import com.supervision.knowsub.vo.dic.DicReqVo;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
@RequiredArgsConstructor
public class DicManageServiceImpl implements DicManageService {

    private final SystemDicService systemDicService;

    private final SystemDicDataService systemDicDataService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String saveDic(DicReqVo dicReqVo) {

        Assert.notEmpty(dicReqVo.getDicName(), "字典名称不能为空");
        Assert.notEmpty(dicReqVo.getDicCode(), "字典编码不能为空");

        Long count = systemDicService.lambdaQuery().eq(SystemDic::getDicCode, dicReqVo.getDicCode()).count();
        Assert.isTrue(count == 0, "字典编码已存在");

        SystemDic systemDic = new SystemDic();
        systemDic.setDicName(dicReqVo.getDicName());
        systemDic.setDicCode(dicReqVo.getDicCode());
        systemDic.setRemark(dicReqVo.getRemark());
        systemDicService.save(systemDic);
        return systemDic.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateDic(DicReqVo dicReqVo) {
        Assert.notEmpty(dicReqVo.getId(), "字典id不能为空");
        Assert.notEmpty(dicReqVo.getDicName(), "字典名称不能为空");
        Assert.notEmpty(dicReqVo.getDicCode(), "字典编码不能为空");


        SystemDic dbDic = systemDicService.getById(dicReqVo.getId());
        Assert.notNull(dbDic, "字典不存在");
        if (!StrUtil.equals(dbDic.getDicCode(), dicReqVo.getDicCode())){
            // 判断字典编码是否重复
            Long count = systemDicService.lambdaQuery().eq(SystemDic::getDicCode, dicReqVo.getDicCode()).count();
            Assert.isTrue(count == 0, "字典编码已存在");
        }

        // 更新数据
        systemDicService.lambdaUpdate().set(SystemDic::getDicName, dicReqVo.getDicName())
                .set(SystemDic::getDicCode, dicReqVo.getDicCode())
                .set(SystemDic::getRemark, dicReqVo.getRemark())
                .eq(SystemDic::getId, dicReqVo.getId()).update();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean deleteDic(String id) {

        Assert.notEmpty(id, "字典id不能为空");

        boolean success = systemDicService.removeById(id);
        if (success){
            systemDicDataService.lambdaUpdate().eq(SystemDicData::getDicId, id).remove();
        }

        return success;
    }

    @Override
    public IPage<SystemDic> listDic(String dicName, String dicCode, Integer pageNum, Integer pageSize) {

        return systemDicService.lambdaQuery()
                .like(StrUtil.isNotEmpty(dicName), SystemDic::getDicName, dicName)
                .eq(StrUtil.isNotEmpty(dicCode), SystemDic::getDicCode, dicCode)
                .orderBy(true,false, SystemDic::getUpdateTime)
                .page(new Page<>(pageNum, pageSize));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String saveDicData(String dicId, List<DicDataReqVo> dicDataReqVoList) {

        Assert.notEmpty(dicId, "字典id不能为空");

        //清除原本的数据
        systemDicDataService.lambdaUpdate().eq(SystemDicData::getDicId, dicId).remove();

        List<SystemDicData> list = dicDataReqVoList.stream().map(dicDataReqVo -> {
            SystemDicData systemDicData = new SystemDicData();
            systemDicData.setDicId(dicId);
            systemDicData.setDicLabel(dicDataReqVo.getDicLabel());
            systemDicData.setDicValue(dicDataReqVo.getDicValue());
            systemDicData.setDicSort(ObjUtil.isNull(dicDataReqVo.getDicSort()) ? 0 : dicDataReqVo.getDicSort());
            return systemDicData;
        }).toList();
        systemDicDataService.saveBatch(list);
        return dicId;
    }

    @Override
    public List<SystemDicData>  queryDicDate(String dicId) {

        Assert.notEmpty(dicId, "字典id不能为空");

        return systemDicDataService.lambdaQuery().eq(SystemDicData::getDicId, dicId).orderBy(true,true, SystemDicData::getDicSort).list();

    }
}
